#!/usr/bin/python
#
# Adds a new user to a Samba4 server
# Copyright Jelmer Vernooij 2008
#
# Based on the original in EJS:
# Copyright Andrew Tridgell 2005
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import samba.getopt as options
from samba.netcmd import Command, CommandError, Option
import ldb

from getpass import getpass
from samba.auth import system_session
from samba.samdb import SamDB
from samba.dsdb import (
    GTYPE_SECURITY_DOMAIN_LOCAL_GROUP,
    GTYPE_SECURITY_GLOBAL_GROUP,
    GTYPE_SECURITY_UNIVERSAL_GROUP,
    GTYPE_DISTRIBUTION_DOMAIN_LOCAL_GROUP,
    GTYPE_DISTRIBUTION_GLOBAL_GROUP,
    GTYPE_DISTRIBUTION_UNIVERSAL_GROUP,
)

securityGroup = dict({"Domain local": GTYPE_SECURITY_DOMAIN_LOCAL_GROUP, "Global": GTYPE_SECURITY_GLOBAL_GROUP, "Universal": GTYPE_SECURITY_UNIVERSAL_GROUP})
distributionGroup = {"Domain local": GTYPE_DISTRIBUTION_DOMAIN_LOCAL_GROUP, "Global": GTYPE_DISTRIBUTION_GLOBAL_GROUP, "Universal": GTYPE_DISTRIBUTION_UNIVERSAL_GROUP}


class cmd_newgroupadv(Command):
    """Creates a new group with additional options"""

    synopsis = "newgroupadv [options] <groupname>"

    takes_optiongroups = {
        "sambaopts": options.SambaOptions,
        "versionopts": options.VersionOptions,
        "credopts": options.CredentialsOptions,
    }

    takes_options = [
        Option("-H", help="LDB URL for database or target server", type=str),
	Option("--GroupOU",
	   help="Alternative location (without domainDN counterpart) to default CN=Users in which new user object will be created", 
	   type=str),
	Option("--groupScope", type="choice", choices=["Domain local", "Global", "Universal"],
            help="Group scope (Domain local | Global | Universal)"),
	Option("--groupType", type="choice", choices=["Security", "Distribution"],
            help="Group type (Security | Distribution)"),
	Option("--description", help="Group's description", type=str),
	Option("--mailAddress", help="Group's email address", type=str),
	Option("--notes", help="Groups's notes", type=str),
    ]

    takes_args = ["groupname"]

    def run(self, groupname, credopts=None, sambaopts=None,
            versionopts=None, H=None, GroupOU=None, groupScope=None,
	    groupType=None, description=None, mailAddress=None, notes=None):

	if cmp((groupType or "Security"), "Security") == 0:
		gType = securityGroup.get(groupScope, GTYPE_SECURITY_GLOBAL_GROUP)
	else:		
		gType = distributionGroup.get(groupScope, GTYPE_DISTRIBUTION_GLOBAL_GROUP)

        lp = sambaopts.get_loadparm()
        creds = credopts.get_credentials(lp)

        try:
            samdb = SamDB(url=H, session_info=system_session(),
                          credentials=creds, lp=lp)
            samdb.newgroupadv(groupname, groupou=GroupOU, grouptype = gType, 
			  description=description, mailaddress=mailAddress, notes=notes)
        except ldb.LdbError, (num, msg):
            raise CommandError('Failed to create user "%s" : %s' % (
                username, msg))

